@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,8 +29,20 @@ import org.apfloat.ApfloatRuntimeException;
 import org.apfloat.spi.ArrayAccess;
 
 /**
- * Kernel for the <code>int</code> element type. Contains everything needed for the NTT.
- * The data is organized in columns, not rows, for efficient processing on the GPU.<p>
+ * Kernel for the <code>int</code> element type. Contains everything needed for the NTT.<p>
+ *
+ * There are two ways the data can be organized:
+ * <ul>
+ *   <li>In columns: this is more efficient on older GPUs that don't have a cache</li>
+ *   <li>In rows: this is more efficient on newer GPUs that have large high-bandwidth caches</li>
+ * </ul>
+ *
+ * When the data is organized in columns, the NTT algorithm makes O(log n) passes through
+ * the main memory of the GPU.<p>
+ *
+ * When the data is organized in rows, and the GPU cache is sufficiently large, the algorithm
+ * makes a fixed number of passes through the main memory of the GPU (and O(log n) passes
+ * through the GPU cache.)<p>
  *
  * Due to the extreme parallelization requirements (global size should be at lest 1024)
  * this algorithm works efficiently only with 4 million decimal digit calculations or bigger.
@@ -53,7 +65,7 @@ import org.apfloat.spi.ArrayAccess;
  * </ul>
  *
  * @since 1.8.3
- * @version 1.9.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -72,8 +84,11 @@ class IntKernel
     private static ThreadLocal<IntKernel> kernel = ThreadLocal.withInitial(IntKernel::new);
 
     // Methods for calculating the column transforms in parallel
-    public static final int TRANSFORM_ROWS = 1;
-    public static final int INVERSE_TRANSFORM_ROWS = 2;
+    public static final int TRANSFORM_ROWS_COLUMNORIENTATION = 1;
+    public static final int INVERSE_TRANSFORM_ROWS_COLUMNORIENTATION = 2;
+    // Methods for calculating the row transforms in parallel
+    public static final int TRANSFORM_ROWS_ROWORIENTATION = 100001;
+    public static final int INVERSE_TRANSFORM_ROWS_ROWORIENTATION = 100002;
 
     public void setLength(int length)
     {
@@ -155,6 +170,80 @@ class IntKernel
         }
     }
 
+    private void rowTableFNT()
+    {
+        int nn, offset, istepbits, mmax, mmaxbits, mmaxmask, r;
+
+        int[] data = this.data;
+        offset = this.offset + getGlobalId(1) * this.length;
+        nn     = this.length;
+
+        if (nn >= 2)
+        {
+            r = 1;
+            mmax = nn >> 1;
+            mmaxbits = numberOfTrailingZeros(mmax);
+            while (mmax > 0)
+            {
+                istepbits = mmaxbits + 1;
+                mmaxmask = mmax - 1;
+
+                /*
+                int t = 0;
+
+                for (int m = 0; m < mmax; m++)
+                {
+                    for (int i = offset + m; i < offset + nn; i += istep)
+                    {
+                        int j = i + mmax;
+                        int a = data[i];
+                        int b = data[j];
+                        data[i] = modAdd(a, b);
+                        data[j] = modMultiply(this.wTable[t], modSubtract(a, b));
+                    }
+                    t += r;
+                }
+                */
+                for (int k = getGlobalId(0); k < nn / 2; k += getGlobalSize(0))
+                {
+                    int m = k & mmaxmask;
+                    int t = m * r;
+                    int i = offset + m + ((k >> mmaxbits) << istepbits);
+                    int j = i + mmax;
+                    int a = data[i];
+                    int b = data[j];
+                    data[i] = modAdd(a, b);
+                    data[j] = modMultiply(wTable[t], modSubtract(a, b));
+                }
+                r <<= 1;
+                mmax >>= 1;
+                mmaxbits--;
+                localBarrier(); // Use a local barrier to synchronize memory writes within the same work-group but don't force memory to be written from the GPU cache to the GPU global memory
+            }
+
+            if (this.permutationTableLength > 0)
+            {
+                rowScramble(offset);
+            }
+        }
+    }
+
+    // Copied from Integer.numberOfTrailingZeros()
+    private int numberOfTrailingZeros(int i)
+    {
+        i = ~i & (i - 1);
+        if (i <= 0)
+        {
+            return i & 32;
+        }
+        int n = 1;
+        if (i > 1 << 16) { n += 16; i >>>= 16; }
+        if (i > 1 <<  8) { n +=  8; i >>>=  8; }
+        if (i > 1 <<  4) { n +=  4; i >>>=  4; }
+        if (i > 1 <<  2) { n +=  2; i >>>=  2; }
+        return n + (i >>> 1);
+    }
+
     private void inverseColumnTableFNT()
     {
         int nn, istep = 0, mmax = 0, r = 0;
@@ -206,6 +295,62 @@ class IntKernel
         }
     }
 
+    private void inverseRowTableFNT()
+    {
+        int nn, offset, istepbits, mmax, mmaxbits, mmaxmask, r;
+
+        int[] data = this.data;
+        offset = this.offset + getGlobalId(1) * this.length;
+        nn     = this.length;
+
+        if (nn >= 2)
+        {
+            if (permutationTableLength > 0)
+            {
+                rowScramble(offset);
+            }
+
+            r = nn;
+            mmax = 1;
+            mmaxbits = 0;
+            while (nn > mmax)
+            {
+                istepbits = mmaxbits + 1;
+                mmaxmask = mmax - 1;
+                r >>= 1;
+
+                /*
+                int t = 0;
+
+                for (int m = 0; m < mmax; m++)
+                {
+                    for (int i = offset + m; i < offset + nn; i += istep)
+                    {
+                        int j = i + mmax;
+                        int wTemp = modMultiply(this.wTable[t], data[j]);
+                        data[j] = modSubtract(data[i], wTemp);
+                        data[i] = modAdd(data[i], wTemp);
+                    }
+                    t += r;
+                }
+                */
+                for (int k = getGlobalId(0); k < nn / 2; k += getGlobalSize(0))
+                {
+                    int m = k & mmaxmask;
+                    int t = m * r;
+                    int i = offset + m + ((k >> mmaxbits) << istepbits);
+                    int j = i + mmax;
+                    int wTemp = modMultiply(wTable[t], data[j]);
+                    data[j] = modSubtract(data[i], wTemp);
+                    data[i] = modAdd(data[i], wTemp);
+                }
+                mmax <<= 1;
+                mmaxbits++;
+                localBarrier(); // Use a local barrier to synchronize memory writes within the same work-group but don't force memory to be written from the GPU cache to the GPU global memory
+            }
+        }
+    }
+
     private void columnScramble(int offset)
     {
         for (int k = 0; k < this.permutationTableLength; k += 2)
@@ -218,6 +363,19 @@ class IntKernel
         }
     }
 
+    private void rowScramble(int offset)
+    {
+        for (int k = 2 * getGlobalId(0); k < this.permutationTableLength; k += 2 * getGlobalSize(0))
+        {
+            int i = offset + this.permutationTable[k],
+                j = offset + this.permutationTable[k + 1];
+            int tmp = this.data[i];
+            this.data[i] = this.data[j];
+            this.data[j] = tmp;
+        }
+        localBarrier();
+    }
+
     private int modMultiply(int a, int b)
     {
         long t = (long) a * (long) b;
@@ -441,14 +599,22 @@ class IntKernel
     @Override
     public void run()
     {
-        if (this.op == TRANSFORM_ROWS)
+        if (this.op == TRANSFORM_ROWS_COLUMNORIENTATION)
         {
             columnTableFNT();
         }
-        else if (this.op == INVERSE_TRANSFORM_ROWS)
+        else if (this.op == INVERSE_TRANSFORM_ROWS_COLUMNORIENTATION)
         {
             inverseColumnTableFNT();
         }
+        else if (this.op == TRANSFORM_ROWS_ROWORIENTATION)
+        {
+            rowTableFNT();
+        }
+        else if (this.op == INVERSE_TRANSFORM_ROWS_ROWORIENTATION)
+        {
+            inverseRowTableFNT();
+        }
         else if (this.op == TRANSPOSE)
         {
             transpose();
