@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -98,7 +98,7 @@ public class LongAparapiMatrixStrategy
         kernel.setArrayAccess(arrayAccess);
         kernel.setN2(n2);
 
-        Range range = Range.create2D(n1, n1);
+        Range range = RangeHelper.create2D(n1, n1);
         kernel.execute(range);
     }
 
@@ -161,7 +161,7 @@ public class LongAparapiMatrixStrategy
 
         kernel.put(index);
 
-        Range range = Range.create(n2 / 2);
+        Range range = RangeHelper.create(n2 / 2);
         kernel.execute(range);
     }
 
@@ -228,7 +228,7 @@ public class LongAparapiMatrixStrategy
 
         kernel.put(index);
 
-        Range range = Range.create(n2);
+        Range range = RangeHelper.create(n2);
         kernel.execute(range);
     }
 }
