@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,11 +28,14 @@ import java.io.ByteArrayOutputStream;
 import java.io.InputStream;
 import java.io.PrintStream;
 
+import org.apfloat.ApfloatArithmeticException;
+import org.apfloat.ApfloatRuntimeException;
+
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -116,7 +119,7 @@ public class CalculatorTest
         {
             // OK: syntax error
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result is not a number
         }
@@ -124,7 +127,7 @@ public class CalculatorTest
         {
             // OK: invalid parameter
         }
-        catch (org.apfloat.ApfloatRuntimeException are)
+        catch (ApfloatRuntimeException are)
         {
             // OK: attempt invalid calculation
         }
@@ -215,6 +218,7 @@ public class CalculatorTest
         assertCalculation("-2.52405e-1", "airyAiPrime(0.200000)");
         assertCalculation("7.05464e-1", "airyBi(0.200000)");
         assertCalculation("4.61789e-1", "airyBiPrime(0.200000)");
+        assertCalculation("9.5235e-1", "angerJ(0.200000,0.700000)");
         assertCalculation("7/6", "bernoulli(14)");
         assertCalculation("3.3124e4", "bernoulliB(4,14.0000)");
         assertCalculation("1.72763", "besselI(0.700000,1.90000)");
@@ -251,6 +255,9 @@ public class CalculatorTest
         assertCalculation("-1.02221", "coshIntegral(0.200000)");
         assertCalculation("1.7061", "digamma(6.0000)");
         assertCalculation("34459425", "doubleFactorial(17)");
+        assertCalculation("1", "doubleFactorial(-1)");
+        assertCalculation("-1", "doubleFactorial(-3)");
+        assertCalculation("1/3", "doubleFactorial(-5)");
         assertCalculation("1.48904", "ellipticE(0.200000)");
         assertCalculation("1.65962", "ellipticK(0.200000)");
         assertCalculation("2.22703e-1", "erf(0.200000)");
@@ -275,7 +282,7 @@ public class CalculatorTest
         assertCalculation("-2.60129", "gegenbauerC(-1.10000,-1.30000,2.50000)");
         assertCalculation("1.282427", "glaisher(7)");
         assertCalculation("1.12151", "harmonicNumber(1.20000)");
-        assertCalculation("1.104112", "harmonicNumber(1.200000,1.300000)");
+        assertCalculation("1.104113", "harmonicNumber(1.200000,1.300000)");
         assertCalculation("42142223/12252240", "harmonicNumber(17)");
         assertCalculation("6301272372663207205033976933/6076911214672415134617600000", "harmonicNumber(13,5)");
         assertCalculation("3.04424", "hermiteH(1.20000,1.30000)");
@@ -304,7 +311,7 @@ public class CalculatorTest
         assertCalculation("1.07204", "legendreQ(1.70000,-0.900000)");
         assertCalculation("4.1358", "legendreQ(1.20000,1.70000,0.900000)");
         assertCalculation("-4.371-3.651i", "logGamma(-1.234+2.345i)");
-        assertCalculation("1.04516", "logIntegral(2.000000)");
+        assertCalculation("1.04516", "logIntegral(2.0000)");
         assertCalculation("8.80797e-1", "logisticSigmoid(2.00000)");
         assertCalculation("1.23", "n(1.23456, 3)");
         assertCalculation("1.04748", "pochhammer(1.20000,0.700000)");
@@ -358,11 +365,14 @@ public class CalculatorTest
         assertCalculation("2/3", "sqrt(4/9)");
         assertCalculation("7.07+7.07i", "sqrt(100.i)");
         assertCalculation("1.4142", "sqrt(2.0000)");
+        assertCalculation("1.45336", "struveH(1.80000,4.70000)");
+        assertCalculation("1.40023", "struveL(1.80000,2.70000)");
         assertCalculation("2", "truncate(2.5)");
         assertCalculation("2", "truncate(5/2)");
         assertCalculation("1.79e2", "toDegrees(3.14)");
         assertCalculation("1.57", "toRadians(90.0)");
-        assertCalculation("1.644", "zeta(2.000)");
+        assertCalculation("2.16872e-1", "weberE(1.700000,0.7000000)");
+        assertCalculation("1.645", "zeta(2.000)");
         assertCalculation("1.181e-1", "zeta(3.000,-1.500)");
         assertCalculation("3.14", "acos(-1.00)");
         assertCalculation("1.01", "acosh(1.55)");
@@ -427,6 +437,9 @@ public class CalculatorTest
         assertCalculationFailure("airyBi(0,0)");
         assertCalculationFailure("airyBiPrime()");
         assertCalculationFailure("airyBiPrime(0,0)");
+        assertCalculationFailure("angerJ()");
+        assertCalculationFailure("angerJ(1.0)");
+        assertCalculationFailure("angerJ(1.0,1.0,1.0)");
         assertCalculationFailure("bernoulli(-1)");
         assertCalculationFailure("bernoulli(i)");
         assertCalculationFailure("bernoulli()");
@@ -481,6 +494,7 @@ public class CalculatorTest
         assertCalculationFailure("digamma()");
         assertCalculationFailure("digamma(2,2)");
         assertCalculationFailure("doubleFactorial(0.5)");
+        assertCalculationFailure("doubleFactorial(-2)");
         assertCalculationFailure("doubleFactorial(2i)");
         assertCalculationFailure("doubleFactorial(2/3)");
         assertCalculationFailure("doubleFactorial()");
@@ -662,6 +676,14 @@ public class CalculatorTest
         assertCalculationFailure("sqrt(2)");
         assertCalculationFailure("sqrt()");
         assertCalculationFailure("sqrt(4, 2)");
+        assertCalculationFailure("struveH()");
+        assertCalculationFailure("struveH(1.0)");
+        assertCalculationFailure("struveH(-1.0,0)");
+        assertCalculationFailure("struveH(1.0,1.0,1.0)");
+        assertCalculationFailure("struveL()");
+        assertCalculationFailure("struveL(1.0)");
+        assertCalculationFailure("struveL(-1.0,0)");
+        assertCalculationFailure("struveL(1.0,1.0,1.0)");
         assertCalculationFailure("truncate(i)");
         assertCalculationFailure("truncate()");
         assertCalculationFailure("truncate(2, 2)");
@@ -673,6 +695,9 @@ public class CalculatorTest
         assertCalculationFailure("toRadians(i)");
         assertCalculationFailure("toRadians()");
         assertCalculationFailure("toRadians(2, 2)");
+        assertCalculationFailure("weberE()");
+        assertCalculationFailure("weberE(1.0)");
+        assertCalculationFailure("weberE(1.0,1.0,1.0)");
         assertCalculationFailure("zeta(1)");
         assertCalculationFailure("zeta(2,0)");
         assertCalculationFailure("zeta(2,-1)");
