@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,7 +44,7 @@ import static java.util.FormattableFlags.*;
  * @see Apfloat
  * @see ApcomplexMath
  *
- * @version 1.13.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -274,7 +274,7 @@ public class Apcomplex
 
     public int radix()
     {
-        return (real().signum() == 0 ? (imag().signum() == 0 ? real().radix() : imag().radix()) : real().radix());
+        return (real().signum() == 0 && imag().signum() != 0 ? imag().radix() : real().radix());
     }
 
     /**
@@ -520,7 +520,7 @@ public class Apcomplex
     {
         if (z.isZero())
         {
-            throw new ArithmeticException(isZero() ? "Zero divided by zero" : "Division by zero");
+            throw new ApfloatArithmeticException(isZero() ? "Zero divided by zero" : "Division by zero", isZero() ? "divide.zeroByZero" : "divide.byZero");
         }
 
         Apfloat tmpReal,
@@ -585,6 +585,7 @@ public class Apcomplex
     /**
      * Returns the value of the this number as a <code>double</code>.
      * Only takes the real part of this number.
+     * The result should be correct only within ± one ulp.
      *
      * @see Apfloat#doubleValue()
      *
@@ -600,6 +601,7 @@ public class Apcomplex
     /**
      * Returns the value of the this number as a <code>float</code>.
      * Only takes the real part of this number.
+     * The result should be correct only within ± one ulp.
      *
      * @see Apfloat#floatValue()
      *
@@ -691,7 +693,7 @@ public class Apcomplex
         long value = longValueExact();
         if (value > Byte.MAX_VALUE || value < Byte.MIN_VALUE)
         {
-            throw new ArithmeticException("Out of range");
+            throw new ApfloatArithmeticException("Out of range", "outOfRange");
         }
         return (byte) value;
     }
@@ -715,7 +717,7 @@ public class Apcomplex
         long value = longValueExact();
         if (value > Short.MAX_VALUE || value < Short.MIN_VALUE)
         {
-            throw new ArithmeticException("Out of range");
+            throw new ApfloatArithmeticException("Out of range", "outOfRange");
         }
         return (short) value;
     }
@@ -739,7 +741,7 @@ public class Apcomplex
         long value = longValueExact();
         if (value > Integer.MAX_VALUE || value < Integer.MIN_VALUE)
         {
-            throw new ArithmeticException("Out of range");
+            throw new ApfloatArithmeticException("Out of range", "outOfRange");
         }
         return (int) value;
     }
@@ -762,7 +764,7 @@ public class Apcomplex
     {
         if (imag().signum() != 0)
         {
-            throw new ArithmeticException("Out of range");
+            throw new ApfloatArithmeticException("Out of range", "outOfRange");
         }
         return real().longValueExact();
     }
