@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -55,7 +55,7 @@ import org.apfloat.spi.ApfloatImpl;
  *
  * @see ApfloatMath
  *
- * @version 1.11.1
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -835,7 +835,7 @@ public class Apfloat
     {
         if (x.signum() == 0)
         {
-            throw new ArithmeticException(signum() == 0 ? "Zero divided by zero" : "Division by zero");
+            throw new ApfloatArithmeticException(signum() == 0 ? "Zero divided by zero" : "Division by zero", signum() == 0 ? "divide.zeroByZero" : "divide.byZero");
         }
         else if (signum() == 0)
         {
@@ -886,6 +886,18 @@ public class Apfloat
         return ApfloatMath.fmod(this, x);
     }
 
+    Apfloat modShort(Apfloat x)
+        throws ApfloatRuntimeException
+    {
+        assert (x.isShort());
+
+        ApfloatImpl thisImpl = getImpl(),
+                    xImpl = x.getImpl(),
+                    impl = thisImpl.modShort(xImpl);
+
+        return new Apfloat(impl);
+    }
+
     /**
      * Floor function. Returns the largest (closest to positive infinity) value
      * that is not greater than this apfloat and is equal to a mathematical integer.
@@ -1060,13 +1072,13 @@ public class Apfloat
     {
         if (!isInteger())
         {
-            throw new ArithmeticException("Rounding necessary");
+            throw new ApfloatArithmeticException("Rounding necessary", "roundingNecessary");
         }
 
         long value = longValue();
         if (!new Apint(value, radix()).equals(truncate()))
         {
-            throw new ArithmeticException("Out of range");
+            throw new ApfloatArithmeticException("Out of range", "outOfRange");
         }
 
         return value;
@@ -1109,6 +1121,8 @@ public class Apfloat
      *
      * @param radix The radix.
      *
+     * @return This number in the specified radix.
+     *
      * @exception NumberFormatException If the radix is invalid.
      *
      * @since 1.2
