@@ -31,6 +31,7 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.ResourceBundle;
 import java.util.MissingResourceException;
+import java.util.ServiceConfigurationError;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.ForkJoinPool;
@@ -192,7 +193,7 @@ import org.apfloat.spi.Util;
  * If these features are added to the Java platform in the future, they
  * may be added to the <code>ApfloatContext</code> API as well.
  *
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -304,10 +305,11 @@ public class ApfloatContext
         @Override
         public void run()
         {
+            ApfloatContext.threadContexts.values().stream().map(ApfloatContext::getExecutorService).forEach(this::shutdown);
+            shutdown(ApfloatContext.globalContext.getExecutorService());
             ApfloatMath.cleanUp();      // Clear references to static cached apfloats
             System.gc();
             System.gc();
-            System.runFinalization();
             this.builderFactory.shutdown();
         }
 
@@ -316,6 +318,21 @@ public class ApfloatContext
             this.builderFactory = builderFactory;
         }
 
+        private void shutdown(ExecutorService executorService)
+        {
+            if (executorService != null)
+            {
+                try
+                {
+                    executorService.shutdownNow();
+                }
+                catch (SecurityException se)
+                {
+                    // Ignore
+                }
+            }
+        }
+
         private BuilderFactory builderFactory;
     }
 
@@ -1028,7 +1045,7 @@ public class ApfloatContext
         }
         catch (Exception e)
         {
-            throw new ApfloatConfigurationException("Error setting property \"" + propertyName + "\" to value \"" + propertyValue + '\"', e);
+            throw new ApfloatConfigurationException("Error setting property \"" + propertyName + "\" to value \"" + propertyValue + '\"', e, "configurationProperty", propertyName, propertyValue);
         }
     }
 
@@ -1155,7 +1172,7 @@ public class ApfloatContext
     {
         if (Thread.interrupted())
         {
-            throw new ApfloatInterruptedException("Interrupted");
+            throw new ApfloatInterruptedException("Interrupted", "interrupted");
         }
     }
 
@@ -1374,20 +1391,25 @@ public class ApfloatContext
 
     static
     {
+        Properties globalProperties = loadSystemOverrides(loadProperties());
+
         ApfloatContext.defaultProperties = new Properties();
 
         // Try to use up to 80% of total memory and all processors
-        long totalMemory;
-        try
+        long totalMemory = Runtime.getRuntime().maxMemory();
+        boolean disableJmx = Boolean.parseBoolean(globalProperties.getProperty("disableJmx", System.getProperty("apfloat.disableJmx")));
+        if (!disableJmx)
         {
-            MemoryMXBean memoryBean = ManagementFactory.getMemoryMXBean();
-            MemoryUsage memoryUsage = memoryBean.getHeapMemoryUsage();
-            totalMemory = Math.max(memoryUsage.getCommitted(), memoryUsage.getMax());
-        }
-        catch (NoClassDefFoundError | NullPointerException e)
-        {
-            // The ManagementFactory class might be unavailable
-            totalMemory = Runtime.getRuntime().maxMemory();
+            try
+            {
+                MemoryMXBean memoryBean = ManagementFactory.getMemoryMXBean();
+                MemoryUsage memoryUsage = memoryBean.getHeapMemoryUsage();
+                totalMemory = Math.max(memoryUsage.getCommitted(), memoryUsage.getMax());
+            }
+            catch (NoClassDefFoundError | ServiceConfigurationError | RuntimeException e)
+            {
+                // The ManagementFactory class might be unavailable
+            }
         }
 
         long maxMemoryBlockSize = Util.round23down(totalMemory / 5 * 4);
@@ -1412,7 +1434,7 @@ public class ApfloatContext
         loadSystemOverrides(ApfloatContext.defaultProperties);
 
         // Set combination of default properties and properties specified in the resource bundle, with system property overrides
-        ApfloatContext.globalContext = new ApfloatContext(loadSystemOverrides(loadProperties()));
+        ApfloatContext.globalContext = new ApfloatContext(globalProperties);
 
         // ExecutorService depends on the properties so set it last
         ApfloatContext.defaultExecutorService = getDefaultExecutorService();
