@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,7 +41,7 @@ import static org.apfloat.spi.RadixConstants.*;
 /**
  * Various utility methods related to apfloats.
  *
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -225,7 +225,7 @@ class ApfloatHelper
         }
         catch (IOException ioe)
         {
-            throw new ApfloatRuntimeException("Should not occur", ioe);
+            throw new ApfloatRuntimeException("Should not occur", ioe, "shouldNotOccur");
         }
         precision = (precision == Apfloat.DEFAULT ? Apfloat.INFINITE : precision);
         return a.toRadix(radix).getImpl(precision);
@@ -406,7 +406,7 @@ class ApfloatHelper
     {
         if (targetPrecision == Apfloat.INFINITE)
         {
-            throw new InfiniteExpansionException("Cannot calculate power to infinite precision");
+            throw new InfiniteExpansionException("Cannot calculate power to infinite precision", "pow.infinitePrecision");
         }
     }
 
@@ -417,7 +417,7 @@ class ApfloatHelper
         {
             if (z.isZero())
             {
-                throw new ArithmeticException("Zero to power zero");
+                throw new ApfloatArithmeticException("Zero to power zero", "pow.zeroToZero");
             }
 
             return new Apcomplex(new Apfloat(1, Apfloat.INFINITE, z.radix()));
@@ -426,7 +426,7 @@ class ApfloatHelper
         {
             if (w.real().signum() <= 0)
             {
-                throw new ArithmeticException("Zero to power of " + (w.imag().signum() == 0 ? "negative number" : "number with nonpositive real part"));
+                throw new ApfloatArithmeticException("Zero to power of " + (w.imag().signum() == 0 ? "negative number" : "number with nonpositive real part"), w.imag().signum() == 0 ? "pow.zeroToNegative" : "pow.zeroToNonpositiveReal");
             }
 
             return z;
@@ -463,7 +463,7 @@ class ApfloatHelper
         }
         else if (x.signum() < 0 && !y.isInteger())
         {
-            throw new ArithmeticException("Power of negative number to non-integer; result would be complex");
+            throw new ApfloatArithmeticException("Power of negative number to non-integer; result would be complex", "pow.negativeToNonInteger");
         }
 
         checkPowPrecision(targetPrecision);
@@ -533,7 +533,7 @@ class ApfloatHelper
         precision = precision - extraPrecision;
         if (precision <= 0)
         {
-            throw new LossOfPrecisionException("Complete loss of precision");
+            throw new LossOfPrecisionException("Complete loss of accurate digits", "lossOfPrecision");
         }
         return precision;
     }
@@ -648,16 +648,31 @@ class ApfloatHelper
     public static Apcomplex reducePrecision(Apcomplex z)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(reducePrecision(z.real()),
-                             reducePrecision(z.imag()));
+        return reducePrecision(z, Apfloat.EXTRA_PRECISION);
     }
 
     // Returns z with precision reduced by specified amount
     public static Apcomplex reducePrecision(Apcomplex z, long extraPrecision)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(reducePrecision(z.real(), extraPrecision),
-                             reducePrecision(z.imag(), extraPrecision));
+        long scale = z.scale();
+        return new Apcomplex(z.real().scale() < scale ? reducePrecisionOrZero(z.real(), extraPrecision) : reducePrecision(z.real(), extraPrecision),
+                             z.imag().scale() < scale ? reducePrecisionOrZero(z.imag(), extraPrecision) : reducePrecision(z.imag(), extraPrecision));
+    }
+
+    private static Apfloat reducePrecisionOrZero(Apfloat x, long extraPrecision)
+    {
+        long precision = x.precision();
+        if (precision == Apfloat.INFINITE)
+        {
+            return x;
+        }
+        precision = precision - extraPrecision;
+        if (precision <= 0)
+        {
+            return Apfloat.ZEROS[x.radix()];
+        }
+        return x.precision(precision);
     }
 
     // Returns z so that gamma(z) should have the given precision 
@@ -747,7 +762,7 @@ class ApfloatHelper
         }
         catch (IOException ioe)
         {
-            throw new ApfloatRuntimeException("Should not occur", ioe);
+            throw new ApfloatRuntimeException("Should not occur", ioe, "shouldNotOccur");
         }
 
         BigInteger b = new BigInteger(x.signum(), bytes);
@@ -822,7 +837,7 @@ class ApfloatHelper
         }
         catch (ArithmeticException ae)
         {
-            throw new OverflowException("Overflow", ae);
+            throw new OverflowException("Overflow", ae, "overflow");
         }
     }
 
