@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,7 @@ import java.util.Random;
 /**
  * Various mathematical functions for arbitrary precision integers.
  *
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -59,10 +59,10 @@ public class ApintMath
         {
             if (x.signum() == 0)
             {
-                throw new ArithmeticException("Zero to power zero");
+                throw new ApfloatArithmeticException("Zero to power zero", "pow.zeroToZero");
             }
 
-            return new Apint(1, x.radix());
+            return Apint.ONES[x.radix()];
         }
         else if (n < 0)
         {
@@ -148,10 +148,10 @@ public class ApintMath
         {
             if (x.signum() == 0)
             {
-                throw new ArithmeticException("Zeroth root of zero");
+                throw new ApfloatArithmeticException("Zeroth root of zero", "root.zerothOfZero");
             }
 
-            Apint one = new Apint(1, x.radix());
+            Apint one = Apint.ONES[x.radix()];
             return new Apint[] { one, x.subtract(one) };
         }
         else if (x.signum() == 0)
@@ -208,7 +208,7 @@ public class ApintMath
     private static Apint powXMinus1(Apint pow, Apint x, long n)
         throws ApfloatRuntimeException
     {
-        Apint one = new Apint(1, x.radix());
+        Apint one = Apint.ONES[x.radix()];
 
         if (n == 2)
         {
@@ -231,7 +231,7 @@ public class ApintMath
     private static Apint powXPlus1(Apint pow, Apint x, long n)
         throws ApfloatRuntimeException
     {
-        Apint one = new Apint(1, x.radix());
+        Apint one = Apint.ONES[x.radix()];
 
         if (n == 2)
         {
@@ -349,7 +349,7 @@ public class ApintMath
     {
         if (y.signum() == 0)
         {
-            throw new ArithmeticException("Division by zero");
+            throw new ApfloatArithmeticException("Division by zero", "divide.byZero");
         }
         else if (x.signum() == 0)
         {
@@ -361,6 +361,10 @@ public class ApintMath
             // x / 1 = x
             return new Apint[] { x, Apint.ZEROS[x.radix()] };
         }
+        else if (y.isShort())
+        {
+            return new Apint[] { x.divide(y), x.mod(y) };
+        }
 
         long precision;
         Apfloat tx, ty;
@@ -517,10 +521,10 @@ public class ApintMath
         {
             if (a.signum() == 0)
             {
-                throw new ArithmeticException("Zero to power zero");
+                throw new ApfloatArithmeticException("Zero to power zero", "pow.zeroToZero");
             }
 
-            return new Apint(1, a.radix());
+            return Apint.ONES[a.radix()];
         }
         else if (m.signum() == 0)
         {
@@ -568,35 +572,30 @@ public class ApintMath
         throws ArithmeticException, ApfloatRuntimeException
     {
         // Extended Euclidean algorithm
-        Apint one = new Apint(1, m.radix()),
+        Apint one = Apint.ONES[m.radix()],
               x = Apint.ZERO,
-              y = one,
               oldX = one,
-              oldY = Apint.ZERO,
               oldA = a,
               b = m;
 
+        // In the algorithm, y is ignored as it's not needed
         while (b.signum() != 0)
         {
-            Apint q = a.divide(b);
+            Apint[] qr = div(a, b);
 
-            Apint tmp = b;
-            b = a.mod(b);
-            a = tmp;
+            Apint q = qr[0];
+            a = b;
+            b = qr[1];
 
-            tmp = x;
+            Apint tmp = x;
             x = oldX.subtract(q.multiply(x));
             oldX = tmp;
-
-            tmp = y;
-            y = oldY.subtract(q.multiply(y));
-            oldY = tmp;
         }
 
         if (!abs(a).equals(one))
         {
             // GCD is not 1
-            throw new ArithmeticException("Modular inverse does not exist");
+            throw new ApfloatArithmeticException("Modular inverse does not exist", "modInverse.notExists");
         }
 
         if (oldX.signum() != oldA.signum())
@@ -663,7 +662,10 @@ public class ApintMath
     public static Apint doubleFactorial(long n)
         throws ArithmeticException, NumberFormatException, ApfloatRuntimeException
     {
-        return new Apint(ApfloatMath.doubleFactorial(n, Apfloat.INFINITE));
+        ApfloatContext ctx = ApfloatContext.getContext();
+        int radix = ctx.getDefaultRadix();
+
+        return doubleFactorial(n, radix);
     }
 
     /**
@@ -683,6 +685,10 @@ public class ApintMath
     public static Apint doubleFactorial(long n, int radix)
         throws ArithmeticException, NumberFormatException, ApfloatRuntimeException
     {
+        if (n < 0)
+        {
+            throw new ApfloatArithmeticException("Double factorial of negative number", "doubleFactorial.ofNegative");
+        }
         return new Apint(ApfloatMath.doubleFactorial(n, Apfloat.INFINITE, radix));
     }
 
@@ -983,7 +989,7 @@ public class ApintMath
         }
         else if (digits == Apfloat.INFINITE)
         {
-            throw new InfiniteExpansionException("Cannot generate an infinite number of random digits");
+            throw new InfiniteExpansionException("Cannot generate an infinite number of random digits", "random.infinitePrecision");
         }
         PushbackReader reader = new PushbackReader(new Reader()
         {
@@ -1016,7 +1022,7 @@ public class ApintMath
         }
         catch (IOException ioe)
         {
-            throw new ApfloatRuntimeException("Error generating random number");
+            throw new ApfloatRuntimeException("Error generating random number", ioe, "random.error");
         }
         
     }
