@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@ import static org.apfloat.ApcomplexMath.isNonPositiveInteger;
  * Helper class for the incomplete gamma function.
  *
  * @since 1.10.0
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -227,7 +227,7 @@ class IncompleteGammaHelper
         {
             if (a.real().signum() <= 0)
             {
-                throw new ArithmeticException("Upper gamma with first argument real part nonpositive and second argment zero");
+                throw new ApfloatArithmeticException("Upper gamma with first argument real part nonpositive and second argment zero", "gammaIncomplete.upperOfNonpositive");
             }
             return ApcomplexMath.gamma(a);
         }
@@ -240,7 +240,7 @@ class IncompleteGammaHelper
     {
         if (a.isZero() && z0.isZero() && z1.isZero())
         {
-            throw new ArithmeticException("Gamma of zero");
+            throw new ApfloatArithmeticException("Gamma of zero", "gamma.ofZero");
         }
         if (z0.equals(z1))
         {
@@ -269,7 +269,7 @@ class IncompleteGammaHelper
         long precision = Arrays.stream(z).mapToLong(Apcomplex::precision).min().getAsLong();
         if (precision == Apfloat.INFINITE)
         {
-            throw new InfiniteExpansionException("Cannot calculate incomplete gamma function to infinite precision");
+            throw new InfiniteExpansionException("Cannot calculate incomplete gamma function to infinite precision", "gammaIncomplete.infinitePrecision");
         }
     }
 
@@ -283,6 +283,11 @@ class IncompleteGammaHelper
         {
             return asymptoticLargeZ(a, z);
         }
+        if (a.isZero() && z.scale() <= 0)
+        {
+            // This comes up in e.g. logIntegral calculation and is faster than the hypergeometric U star algorithm
+            return new GammaValue(a, upperGamma(0, z), false);
+        }
         GammaValue result = attemptUStar(a, z);
         if (result != null)
         {
@@ -331,7 +336,7 @@ class IncompleteGammaHelper
     {
         if (isNonPositiveInteger(a))
         {
-            throw new ArithmeticException("Lower gamma with first argument nonpositive integer");
+            throw new ApfloatArithmeticException("Lower gamma with first argument nonpositive integer", "gammaIncomplete.lowerOfNonpositiveInteger");
         }
         if (useSum(z))
         {
@@ -409,7 +414,7 @@ class IncompleteGammaHelper
     private static Sequence upperGammaSequence(Apcomplex a, Apcomplex z)
     {
         int radix = z.radix();
-        Apfloat one = new Apint(1, radix);
+        Apfloat one = Apint.ONES[radix];
         Apcomplex za = z.subtract(a);
         Sequence s = new Sequence(n ->
         {
@@ -430,7 +435,7 @@ class IncompleteGammaHelper
     private static Sequence lowerGammaSequence(Apcomplex a, Apcomplex z)
     {
         int radix = z.radix();
-        Apfloat one = new Apint(1, radix);
+        Apfloat one = Apint.ONES[radix];
         Sequence s = new Sequence(n ->
         {
             if (n == 1)
@@ -453,7 +458,7 @@ class IncompleteGammaHelper
     private static Sequence upperGammaSequenceAlternative(Apcomplex a, Apcomplex z)
     {
         int radix = z.radix();
-        Apfloat one = new Apint(1, radix);
+        Apfloat one = Apint.ONES[radix];
         Sequence s = new Sequence(n ->
         {
             if (n == 1)
@@ -476,7 +481,7 @@ class IncompleteGammaHelper
     private static Sequence lowerGammaSequenceAlternative(Apcomplex a, Apcomplex z)
     {
         int radix = z.radix();
-        Apfloat one = new Apint(1, radix);
+        Apfloat one = Apint.ONES[radix];
         Apcomplex az = a.add(z);
         Sequence s = new Sequence(n ->
         {
@@ -537,7 +542,7 @@ class IncompleteGammaHelper
                 else if (resultIterations == fastestIterations)
                 {
                     // If neither continued fraction reached the precision goal within the max iterations, see which one got better precision
-                    Apint one = new Apint(1, radix);
+                    Apint one = Apint.ONES[radix];
                     long resultPrecision = result.getDelta().equalDigits(one);
                     long fastestPrecision = fastestResult.getDelta().equalDigits(one);
                     if (resultPrecision > fastestPrecision)
@@ -592,7 +597,7 @@ class IncompleteGammaHelper
     // Modified Lentz's method
     private static ContinuedFractionResult continuedFraction(Sequence s, int radix, long workingPrecision, long minIterations, long maxIterations)
     {
-        Apint one = new Apint(1, radix);
+        Apint one = Apint.ONES[radix];
         long n = 0;
         Apcomplex an;
         Apcomplex bn;
@@ -776,8 +781,6 @@ class IncompleteGammaHelper
     // Exponential integral for upperGamma(0, z)
     private static Apcomplex e1(Apcomplex z)
     {
-        assert (isCloseToNegativeRealAxis(z));
-
         int radix = z.radix();
         long targetPrecision = z.precision();
         Apcomplex mz = ApfloatHelper.extendPrecision(z).negate();
