@@ -30,7 +30,7 @@ package org.apfloat;
  * For example, <code>new Apfloat(2).divide(new Apfloat(3))</code>, in radix 10.
  *
  * @since 1.5
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -68,5 +68,36 @@ public class InfiniteExpansionException
         super(message, cause);
     }
 
+    /**
+     * Constructs a new apfloat infinite expansion exception with the specified detail message and localization data.
+     *
+     * @param message The detail message.
+     * @param localizationKey The localization key.
+     * @param localizationArgs The localization arguments.
+     *
+     * @since 1.15.0
+     */
+
+    public InfiniteExpansionException(String message, String localizationKey, Object... localizationArgs)
+    {
+        super(message, localizationKey, localizationArgs);
+    }
+
+    /**
+     * Constructs a new apfloat infinite expansion exception with the specified detail message, cause and localization data.
+     *
+     * @param message The detail message.
+     * @param cause Originating cause of the exception.
+     * @param localizationKey The localization key.
+     * @param localizationArgs The localization arguments.
+     *
+     * @since 1.15.0
+     */
+
+    public InfiniteExpansionException(String message, Throwable cause, String localizationKey, Object... localizationArgs)
+    {
+        super(message, cause, localizationKey, localizationArgs);
+    }
+
     private static final long serialVersionUID = -7022924635011038776L;
 }
