@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,7 @@ import org.apfloat.spi.Util;
  * Helper class for Lambert W function.
  *
  * @since 1.8.0
- * @version 1.10.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -67,7 +67,7 @@ class LambertWHelper {
         }
         this.k = k;
         this.minusOne = new Apint(-1, this.radix);
-        this.one = new Apint(1, this.radix);
+        this.one = Apint.ONES[this.radix];
         this.two = new Apint(2, this.radix);
         this.three = new Apint(3, this.radix);
         this.minusOnePerE = new Apfloat(BRANCH_POINT_BEYOND, Apfloat.DEFAULT, this.radix);  // Slightly beyond the limit to cater for round-off errors
@@ -85,7 +85,7 @@ class LambertWHelper {
 
         if ((this.z.real().signum() != 0 || this.z.imag().signum() != 0) && this.precision == Apfloat.INFINITE)
         {
-            throw new InfiniteExpansionException("Cannot calculate W to infinite precision");
+            throw new InfiniteExpansionException("Cannot calculate W to infinite precision", "w.infinitePrecision");
         }
     }
 
@@ -138,7 +138,7 @@ class LambertWHelper {
         if (this.x.compareTo(this.minusOnePerE) < 0)
         {
             // We know for sure here, but it could still be, even if we don't notice it here
-            throw new ComplexException(new ArithmeticException("Result would be complex"));
+            throw new ComplexException(new ApfloatArithmeticException("Result would be complex", "complex"));
         }
         else if (this.k == -1 && !this.close)
         {
@@ -217,7 +217,7 @@ class LambertWHelper {
             // If x was close to the limit and we did not notice it earlier, w may now have diverged
             if (this.k == 0 && w.compareTo(this.minusOne) < 0 || this.k == -1 && w.compareTo(this.minusOne) > 0)
             {
-                throw new ComplexException(new ArithmeticException("Result would be complex"));
+                throw new ComplexException(new ApfloatArithmeticException("Result would be complex", "complex"));
             }
 
             // Check the convergence
@@ -247,7 +247,7 @@ class LambertWHelper {
             }
             else
             {
-                throw new ArithmeticException("W_" + this.k + " of zero");
+                throw new ApfloatArithmeticException("W_" + this.k + " of zero", "w.ofZero", this.k);
             }
         }
 
@@ -462,7 +462,7 @@ class LambertWHelper {
     {
         if (p.imag().signum() != 0)
         {
-            throw new ComplexException(new ArithmeticException("Result would be complex"));
+            throw new ComplexException(new ApfloatArithmeticException("Result would be complex", "complex"));
         }
         return complexSeries(p).real();
     }
