@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -58,7 +58,7 @@ import static org.apfloat.internal.RawtypeRadixConstants.*;
  * This implementation doesn't necessarily store any extra digits for added
  * precision, so the last digit of any operation may be inaccurate.
  *
- * @version 1.11.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -260,7 +260,7 @@ public class RawtypeApfloatImpl
         // Check for overflow in exponent as represented in base units
         if (baseExp > MAX_EXPONENT[this.radix])
         {
-            throw new OverflowException("Overflow");
+            throw new OverflowException("Overflow", "overflow");
         }
         else if (baseExp < -MAX_EXPONENT[this.radix])
         {
@@ -786,7 +786,7 @@ public class RawtypeApfloatImpl
         // Check for overflow in exponent as represented in base units
         if (baseExp > MAX_EXPONENT[this.radix])
         {
-            throw new OverflowException("Overflow");
+            throw new OverflowException("Overflow", "overflow");
         }
         else if (baseExp < -MAX_EXPONENT[this.radix])
         {
@@ -889,14 +889,14 @@ public class RawtypeApfloatImpl
     {
         if (!(x instanceof RawtypeApfloatImpl))
         {
-            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName());
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
         }
 
         RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
 
         if (this.radix != that.radix)
         {
-            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix);
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
         }
 
         assert (this.sign != 0);
@@ -956,7 +956,7 @@ public class RawtypeApfloatImpl
 
                 if (this.exponent == MAX_EXPONENT[this.radix] && carrySize > 0)
                 {
-                    throw new OverflowException("Overflow");
+                    throw new OverflowException("Overflow", "overflow");
                 }
 
                 if (precision != Apfloat.INFINITE &&
@@ -1144,7 +1144,7 @@ public class RawtypeApfloatImpl
 
                 if (this.exponent == MAX_EXPONENT[this.radix] && leadingZeros == 0)
                 {
-                    throw new OverflowException("Overflow");
+                    throw new OverflowException("Overflow", "overflow");
                 }
             }
 
@@ -1184,14 +1184,14 @@ public class RawtypeApfloatImpl
     {
         if (!(x instanceof RawtypeApfloatImpl))
         {
-            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName());
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
         }
 
         RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
 
         if (this.radix != that.radix)
         {
-            throw new RadixMismatchException("Cannot multiply numbers with different radixes: " + this.radix + " and " + that.radix);
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
         }
 
         int sign = this.sign * that.sign;
@@ -1205,7 +1205,7 @@ public class RawtypeApfloatImpl
 
         if (exponent > MAX_EXPONENT[this.radix])
         {
-            throw new OverflowException("Overflow");
+            throw new OverflowException("Overflow", "overflow");
         }
         else if (exponent < -MAX_EXPONENT[this.radix])
         {
@@ -1270,18 +1270,19 @@ public class RawtypeApfloatImpl
     {
         if (!(x instanceof RawtypeApfloatImpl))
         {
-            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName());
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
         }
 
         RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
 
         if (this.radix != that.radix)
         {
-            throw new RadixMismatchException("Cannot divide numbers with different radixes: " + this.radix + " and " + that.radix);
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
         }
 
         assert (this.sign != 0);
         assert (that.sign != 0);
+        assert (that.getSize() == 1);
 
         int sign = this.sign * that.sign;
 
@@ -1289,7 +1290,7 @@ public class RawtypeApfloatImpl
 
         if (exponent > MAX_EXPONENT[this.radix])
         {
-            throw new OverflowException("Overflow");
+            throw new OverflowException("Overflow", "overflow");
         }
         else if (exponent < -MAX_EXPONENT[this.radix])
         {
@@ -1299,7 +1300,7 @@ public class RawtypeApfloatImpl
 
         long precision = Math.min(this.precision, that.precision),
              basePrecision = getBasePrecision(),
-             thisDataSize = Math.min(getSize(), basePrecision);
+             thisDataSize = getSize();
 
         DataStorage dataStorage;
 
@@ -1345,7 +1346,7 @@ public class RawtypeApfloatImpl
 
                 if (basePrecision == Apfloat.INFINITE)
                 {
-                    throw new InfiniteExpansionException("Cannot perform inexact division to infinite precision");
+                    throw new InfiniteExpansionException("Cannot perform inexact division to infinite precision", "divide.infinitePrecision");
                 }
 
                 size = basePrecision;
@@ -1354,16 +1355,10 @@ public class RawtypeApfloatImpl
             {
                 // Divisor contains only factors of the base; calculate maximum sequence length
                 carry = (rawtype) 1;
-                DataStorage.Iterator dummy = new DataStorage.Iterator()
-                {
-                    @Override public void setRawtype(rawtype value) {}
-                    @Override public void next() {}
-                    private static final long serialVersionUID = 1L;
-                };
                 long sequenceSize;
                 for (sequenceSize = 0; carry != 0; sequenceSize++)
                 {
-                    carry = additionStrategy.divide(null, divisor, carry, dummy, 1);
+                    carry = additionStrategy.divide(null, divisor, carry, null, 1);
                 }
 
                 size = Math.min(basePrecision, thisDataSize + sequenceSize);
@@ -1404,6 +1399,151 @@ public class RawtypeApfloatImpl
         return new RawtypeApfloatImpl(sign, precision, exponent, dataStorage, this.radix);
     }
 
+    @Override
+    public ApfloatImpl modShort(ApfloatImpl x)
+        throws ApfloatRuntimeException
+    {
+        if (!(x instanceof RawtypeApfloatImpl))
+        {
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
+        }
+
+        RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
+
+        if (this.radix != that.radix)
+        {
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
+        }
+
+        assert (this.sign != 0);
+        assert (that.sign != 0);
+        assert (this.exponent >= that.exponent);
+        assert (that.getSize() == 1);
+
+        rawtype modulus = getMostSignificantWord(that.dataStorage);
+
+        ApfloatContext ctx = ApfloatContext.getContext();
+        AdditionBuilder<RawType> additionBuilder = ctx.getBuilderFactory().getAdditionBuilder(RawType.TYPE);
+        AdditionStrategy<RawType> additionStrategy = additionBuilder.createAddition(this.radix);
+
+        // Check if we need to go through all the digits or just some of the last ones
+        rawtype dividend = modulus;
+
+        // Check that the factorization of the modulus consists entirely of factors of the base
+        // E.g. if base is 10=2*5 then the modulus should be 2^n*5^m
+        for (int i = 0; i < RADIX_FACTORS[this.radix].length; i++)
+        {
+            rawtype factor = RADIX_FACTORS[this.radix][i],
+                    quotient;
+
+            // Keep dividing by factor as long as dividend % factor == 0
+            // that is remove factors of the base from the modulus
+            while ((dividend - factor * (quotient = (rawtype) (long) (dividend / factor))) == 0)
+            {
+                dividend = quotient;
+            }
+        }
+
+        // Check if the modulus was factored all the way to one by just dividing by factors of the base
+
+        long exponentDifference = this.exponent - that.exponent,
+             sequenceSize;
+        if (dividend != (rawtype) 1)
+        {
+            // Modulus does not contain only factors of the base; need to the division to full length
+            sequenceSize = exponentDifference + 1;
+        }
+        else
+        {
+            // Modulus contains only factors of the base; calculate maximum sequence length
+            rawtype carry = (rawtype) 1;
+            for (sequenceSize = 0; carry != 0; sequenceSize++)
+            {
+                carry = additionStrategy.divide(null, modulus, carry, null, 1);
+            }
+        }
+
+        // Possible cases
+
+        // Legend:
+        // Significant digit:     X
+        // Irrelevant digit:      -
+        // Empty (implicit zero):  
+
+        // this:       XXXXXXXX
+        // that:                 X
+
+        // this:       XXXXXXXXXXXXXX
+        // that:                 X
+
+        // this:       -----XXX
+        // that:                 X
+
+        // this:       -----XXXXXXXXX
+        // that:                 X
+
+        // this:       -----
+        // that:                 X
+
+        long thisDataSize = getSize();
+
+        if (thisDataSize <= exponentDifference - sequenceSize)
+        {
+            // this:       -----
+            // that:                 X
+            return zero();
+        }
+
+        long size = Math.max(thisDataSize - exponentDifference, 1),
+             start = Math.max(exponentDifference - sequenceSize + 1, 0),
+             end = Math.min(thisDataSize, exponentDifference + 1);
+
+        DataStorage dataStorage = createDataStorage(size);
+        dataStorage.setSize(size);
+
+        DataStorage.Iterator src = this.dataStorage.iterator(DataStorage.READ, start, thisDataSize);
+
+        // Perform actual division
+        rawtype carry = additionStrategy.divide(src, modulus, (rawtype) 0, null, end - start);
+
+        // Continue division until the remainder is smaller than the modulus
+        carry = additionStrategy.divide(null, modulus, carry, null, exponentDifference - end + 1);
+
+        try (DataStorage.Iterator dst = dataStorage.iterator(DataStorage.WRITE, 0, size))
+        {
+            // Set the carry (most significant word of remainder)
+            dst.setRawtype(carry);
+            dst.next();
+
+            // Copy remaining digits, if any, from this to that
+            additionStrategy.add(src, null, (rawtype) 0, dst, size - 1);
+        }
+
+        long exponent = that.exponent;
+
+        dataStorage.setReadOnly();
+
+        long leadingZeros = getLeadingZeros(dataStorage, 0);
+        if (leadingZeros == size)
+        {
+            // Remainder is zero
+            return zero();
+        }
+        dataStorage = dataStorage.subsequence(leadingZeros, size - leadingZeros);
+        exponent -= leadingZeros;
+
+        if (exponent < -MAX_EXPONENT[this.radix])
+        {
+            // Underflow
+            return zero();
+        }
+
+        long scale = (exponent - 1) * BASE_DIGITS[this.radix] + getInitialDigits(dataStorage),
+             precision = Math.min(Util.ifFinite(precision(), precision() - scale() + scale), Util.ifFinite(that.precision(), that.precision() - that.scale() + scale));
+
+        return new RawtypeApfloatImpl(this.sign, precision, exponent, dataStorage, this.radix);
+    }
+
     @Override
     public ApfloatImpl absFloor()
         throws ApfloatRuntimeException
@@ -1656,28 +1796,29 @@ public class RawtypeApfloatImpl
         }
 
         double value = 0.0,
+               factor = 1.0,
                doubleBase = (double) BASE[this.radix];
 
         int size = (int) Math.min(MAX_DOUBLE_SIZE, getSize());
 
-        DataStorage.Iterator iterator = this.dataStorage.iterator(DataStorage.READ, size, 0);
+        DataStorage.Iterator iterator = this.dataStorage.iterator(DataStorage.READ, 0, size);
 
         while (iterator.hasNext())
         {
-            value += (double) iterator.getRawtype();
-            value /= doubleBase;
+            value += (double) iterator.getRawtype() * factor;
+            factor /= doubleBase;
             iterator.next();
         }
 
-        // If the end result fits in a double, any intermediate calculation must not overflow
-        // Note that 1/BASE <= value < 1
+        // If the end result fits in a double, any intermediate calculation must not overflow or underflow
+        // Note that 1 <= value < BASE
         if (this.exponent > 0)
         {
-            return this.sign * value * Math.pow((double) BASE[this.radix], (double) (this.exponent - 1)) * BASE[this.radix];
+            return this.sign * value * Math.pow((double) BASE[this.radix], (double) (this.exponent - 1));
         }
         else
         {
-            return this.sign * value * Math.pow((double) BASE[this.radix], (double) this.exponent);
+            return this.sign * value * Math.pow((double) BASE[this.radix], (double) this.exponent) / BASE[this.radix];
         }
     }
 
@@ -1751,7 +1892,7 @@ public class RawtypeApfloatImpl
     {
         if (!(x instanceof RawtypeApfloatImpl))
         {
-            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName());
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
         }
 
         RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
@@ -1766,7 +1907,7 @@ public class RawtypeApfloatImpl
         }
         else if (this.radix != that.radix)
         {
-            throw new RadixMismatchException("Cannot compare values with different radixes: " + this.radix + " and " + that.radix);
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
         }
 
         long thisScale = scale(),
@@ -1893,7 +2034,7 @@ public class RawtypeApfloatImpl
     {
         if (!(x instanceof RawtypeApfloatImpl))
         {
-            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName());
+            throw new ImplementationMismatchException("Wrong operand type: " + x.getClass().getName(), "type.mismatch", x.getClass().getName());
         }
 
         RawtypeApfloatImpl that = (RawtypeApfloatImpl) x;
@@ -1912,7 +2053,7 @@ public class RawtypeApfloatImpl
         }
         else if (this.radix != that.radix)
         {
-            throw new RadixMismatchException("Cannot compare values with different radixes: " + this.radix + " and " + that.radix);
+            throw new RadixMismatchException("Cannot use numbers with different radixes: " + this.radix + " and " + that.radix, "radix.mismatch", this.radix, that.radix);
         }
         else if (scale() < that.scale())                // Now we know that both have same sign (which is not zero)
         {
@@ -2183,7 +2324,7 @@ public class RawtypeApfloatImpl
 
         if (length > Integer.MAX_VALUE || length < 0)           // Detect overflow
         {
-            throw new ApfloatInternalException("Number is too large to fit in a String");
+            throw new ApfloatInternalException("Number is too large to fit in a String", "stringSizeExceeded");
         }
 
         StringWriter writer = new StringWriter((int) length);
@@ -2194,7 +2335,7 @@ public class RawtypeApfloatImpl
         }
         catch (IOException ioe)
         {
-            throw new ApfloatInternalException("Unexpected I/O error writing to StringWriter", ioe);
+            throw new ApfloatInternalException("Unexpected I/O error writing to StringWriter", ioe, "string.error");
         }
 
         String value = writer.toString();
