@@ -38,7 +38,7 @@ import java.util.Locale;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.13.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -545,9 +545,10 @@ public class ApcomplexTest
             a.divide(new Apcomplex("0"));
             fail("Division by zero allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: division by zero
+            assertEquals("Localization key", "divide.byZero", aae.getLocalizationKey());
         }
     }
 
@@ -581,9 +582,10 @@ public class ApcomplexTest
             new Apcomplex("5.5").longValueExact();
             fail("Non-integer number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
 
         try
@@ -591,9 +593,10 @@ public class ApcomplexTest
             new Apcomplex("5.5").intValueExact();
             fail("Non-integer number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
 
         try
@@ -601,9 +604,10 @@ public class ApcomplexTest
             new Apcomplex("5.5").shortValueExact();
             fail("Non-integer number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
 
         try
@@ -611,9 +615,10 @@ public class ApcomplexTest
             new Apcomplex("5.5").byteValueExact();
             fail("Non-integer number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
 
         try
@@ -621,9 +626,10 @@ public class ApcomplexTest
             new Apcomplex("(5,6)").longValueExact();
             fail("Nonreal number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "outOfRange", aae.getLocalizationKey());
         }
 
         try
@@ -631,9 +637,10 @@ public class ApcomplexTest
             new Apcomplex("(5,6)").intValueExact();
             fail("Nonreal number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "outOfRange", aae.getLocalizationKey());
         }
 
         try
@@ -641,9 +648,10 @@ public class ApcomplexTest
             new Apcomplex("(5,6)").shortValueExact();
             fail("Nonreal number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "outOfRange", aae.getLocalizationKey());
         }
 
         try
@@ -651,9 +659,10 @@ public class ApcomplexTest
             new Apcomplex("(5,6)").byteValueExact();
             fail("Nonreal number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "outOfRange", aae.getLocalizationKey());
         }
     }
 
@@ -844,8 +853,6 @@ public class ApcomplexTest
     public static void testFormatTo()
         throws IOException
     {
-        System.setProperty("java.locale.providers", "COMPAT,SPI"); // Required since Java 10 to have all locale providers available
-
         Locale locale = null;
         assertEquals("null %s", "(1.23456789e5, 1.23456e2)", String.format(locale, "%s", new Apcomplex("(123456.789,123.456)")));
         assertEquals("null %S", "(1.23456789E5, 1.23456E2)", String.format(locale, "%S", new Apcomplex("(123456.789,123.456)")));
@@ -863,7 +870,7 @@ public class ApcomplexTest
         assertEquals("fi_FI %s", "(1,23456789e5, 1,23456e2)", String.format(locale, "%s", new Apcomplex("(123456.789,123.456)")));
         assertEquals("fi_FI %s radix 11", "(1,23456e5, 1,23456e5)", String.format(locale, "%s", new Apcomplex(new Apfloat("123456", 6, 11), new Apfloat("123456", 6, 11))));
 
-        locale = new Locale("hi", "IN");
+        locale = new Locale.Builder().setLanguage("hi").setRegion("IN").setExtension('u', "nu-deva").build();
         assertEquals("hi_IN %#.6s", "(१२३४५६, १२३४५६)", String.format(locale, "%#.6s", new Apcomplex(new Apfloat("123456.7890123456"), new Apfloat("123456.7890123456"))));
         assertEquals("hi_IN %#s radix 9", "(१२३४५६, १२३४५६)", String.format(locale, "%#s", new Apcomplex(new Apfloat("123456", 6, 9), new Apfloat("123456", 6, 9))));
         assertEquals("hi_IN %s radix 11", "(1.23456e5, 1.23456e5)", String.format(locale, "%s", new Apcomplex(new Apfloat("123456", 6, 11), new Apfloat("123456", 6, 11))));
