@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@ import java.util.Map;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -89,9 +89,10 @@ public class ApintMathTest
             ApintMath.pow(new Apint(0), 0);
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
     }
 
@@ -196,9 +197,10 @@ public class ApintMathTest
             ApintMath.root(new Apint(-2), 2);
             fail("sqrt of -2 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be imaginary
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
 
         try
@@ -206,9 +208,10 @@ public class ApintMathTest
             ApintMath.root(new Apint(0), 0);
             fail("0th root of 0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be undefined
+            assertEquals("Localization key", "root.zerothOfZero", aae.getLocalizationKey());
         }
     }
 
@@ -288,6 +291,11 @@ public class ApintMathTest
         assertEquals("0 / 3", new Apint(0), r[0]);
         assertEquals("0 % 3", new Apint(0), r[1]);
 
+        r = ApintMath.div(new Apint("101010101010101010101010101010101010101010101010101010101010101010101010101010100"),
+                          new Apint("101010101010101010101010101010101010101010101010101010101010101010101010101010101"));
+        assertEquals("101...100 / 101...101", new Apint(0), r[0]);
+        assertEquals("101...100 % 101...101", new Apint("101010101010101010101010101010101010101010101010101010101010101010101010101010100"), r[1]);
+
         r = ApintMath.div(new Apint("101010101010101010101010101010101010101010101010101010101010101010101010101010101"),
                           new Apint("101010101010101010101010101010101010101010101010101010101010101010101010101010101"));
         assertEquals("101...101 / 101...101", new Apint(1), r[0]);
@@ -338,9 +346,10 @@ public class ApintMathTest
             ApintMath.div(new Apint(3), new Apint(0));
             fail("3 div 0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be infinite
+            assertEquals("Localization key", "divide.byZero", aae.getLocalizationKey());
         }
     }
 
@@ -440,9 +449,10 @@ public class ApintMathTest
             ApintMath.modPow(new Apint(14), new Apint(-1), new Apint(65536));
             fail("Non-existent modular inverse allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: modular inverse does not exist
+            assertEquals("Localization key", "modInverse.notExists", aae.getLocalizationKey());
         }
 
         try
@@ -450,9 +460,10 @@ public class ApintMathTest
             ApintMath.modPow(new Apint(0), new Apint(-1), new Apint(2));
             fail("Non-existent modular inverse allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: modular inverse does not exist
+            assertEquals("Localization key", "modInverse.notExists", aae.getLocalizationKey());
         }
 
         try
@@ -460,9 +471,10 @@ public class ApintMathTest
             ApintMath.modPow(new Apint(0), new Apint(0), new Apint(7));
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
     }
 
@@ -484,9 +496,10 @@ public class ApintMathTest
             ApintMath.factorial(-1);
             fail("Factorial of negative number allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: factorial of negative number
+            assertEquals("Localization key", "factorial.ofNegative", aae.getLocalizationKey());
         }
     }
 
@@ -494,7 +507,7 @@ public class ApintMathTest
     {
         assertEquals("0!!", new Apint(1), ApintMath.doubleFactorial(0));
         assertEquals("1!!", new Apint(1), ApintMath.doubleFactorial(1));
-        assertEquals("2!!", new Apint(2), ApintMath.factorial(2));
+        assertEquals("2!!", new Apint(2), ApintMath.doubleFactorial(2));
         assertEquals("3!!", new Apint(3), ApintMath.doubleFactorial(3));
         assertEquals("7!!", new Apint(105), ApintMath.doubleFactorial(7));
         assertEquals("7!! radix", 10, ApintMath.doubleFactorial(7).radix());
@@ -508,9 +521,21 @@ public class ApintMathTest
             ApintMath.doubleFactorial(-1);
             fail("Double factorial of negative number allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: double factorial of negative number
+            assertEquals("Localization key", "doubleFactorial.ofNegative", aae.getLocalizationKey());
+        }
+
+        try
+        {
+            ApintMath.doubleFactorial(-2);
+            fail("Double factorial of negative even number allowed");
+        }
+        catch (ApfloatArithmeticException aae)
+        {
+            // OK: double factorial of negative number
+            assertEquals("Localization key", "doubleFactorial.ofNegative", aae.getLocalizationKey());
         }
     }
 
@@ -662,6 +687,7 @@ public class ApintMathTest
         catch (OverflowException oe)
         {
             // OK: overflow
+            assertEquals("Localization key", "overflow", oe.getLocalizationKey());
         }
     }
 
@@ -759,6 +785,7 @@ public class ApintMathTest
         catch (InfiniteExpansionException iee)
         {
             // OK: Infinite size
+            assertEquals("Localization key", "random.infinitePrecision", iee.getLocalizationKey());
         }
     }
 
