@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,13 +25,11 @@ package org.apfloat;
 
 import java.math.RoundingMode;
 import java.util.Arrays;
-import java.util.Iterator;
-import java.util.NoSuchElementException;
 
 import junit.framework.TestSuite;
 
 /**
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -92,9 +90,10 @@ public class AprationalMathTest
             AprationalMath.pow(new Aprational(new Apint(0)), 0);
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
     }
 
@@ -155,9 +154,10 @@ public class AprationalMathTest
             AprationalMath.roundToPrecision(new Aprational("1/3"), 1000, RoundingMode.UNNECESSARY);
             fail("rounding accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; rounding needed
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
     }
 
@@ -186,9 +186,10 @@ public class AprationalMathTest
             AprationalMath.roundToInteger(new Aprational("1/2"), RoundingMode.UNNECESSARY);
             fail("rounding accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; rounding needed
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
     }
 
@@ -242,9 +243,10 @@ public class AprationalMathTest
             AprationalMath.roundToPlaces(new Aprational("1/3"), 1000, RoundingMode.UNNECESSARY);
             fail("rounding accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; rounding needed
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
     }
 
@@ -309,9 +311,10 @@ public class AprationalMathTest
             AprationalMath.roundToMultiple(new Aprational("1/3"), new Aprational("1/2"), RoundingMode.UNNECESSARY);
             fail("rounding accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; rounding needed
+            assertEquals("Localization key", "roundingNecessary", aae.getLocalizationKey());
         }
 
         try
@@ -319,9 +322,10 @@ public class AprationalMathTest
             AprationalMath.roundToMultiple(new Aprational("1/3"), new Aprational("0"), RoundingMode.UP);
             fail("Non-zero as multiple of zero");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; impossible
+            assertEquals("Localization key", "round.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -505,6 +509,50 @@ public class AprationalMathTest
         assertEquals("min of 2/3 and 1/2", new Aprational("1/2"), AprationalMath.min(new Aprational("2/3"), new Aprational("1/2")));
     }
 
+    public static void testDoubleFactorial()
+    {
+        assertEquals("0!!", new Apint(1), AprationalMath.doubleFactorial(0));
+        assertEquals("1!!", new Apint(1), AprationalMath.doubleFactorial(1));
+        assertEquals("2!!", new Apint(2), AprationalMath.doubleFactorial(2));
+        assertEquals("3!!", new Apint(3), AprationalMath.doubleFactorial(3));
+        assertEquals("7!!", new Apint(105), AprationalMath.doubleFactorial(7));
+        assertEquals("7!! radix", 10, AprationalMath.doubleFactorial(7).radix());
+        assertEquals("7!! radix 7", new Apint(105, 7), AprationalMath.doubleFactorial(7, 7));
+        assertEquals("7!! radix 7 radix", 7, AprationalMath.doubleFactorial(7, 7).radix());
+        assertEquals("20!!", new Apint("3715891200"), AprationalMath.doubleFactorial(20));
+        assertEquals("29!!", new Apint("6190283353629375"), AprationalMath.doubleFactorial(29));
+
+        assertEquals("-1!!", new Aprational("1"), AprationalMath.doubleFactorial(-1));
+        assertEquals("-3!!", new Aprational("-1"), AprationalMath.doubleFactorial(-3));
+        assertEquals("-7!!", new Aprational("-1/15"), AprationalMath.doubleFactorial(-7));
+        assertEquals("-7!! radix", 10, AprationalMath.doubleFactorial(-7).radix());
+        assertEquals("-7!! radix 7", new Aprational(new Apint(-1, 7), new Apint(15, 7)), AprationalMath.doubleFactorial(-7, 7));
+        assertEquals("-7!! radix 7 radix", 7, AprationalMath.doubleFactorial(-7, 7).radix());
+        assertEquals("-29!!", new Aprational("1/213458046676875"), AprationalMath.doubleFactorial(-29));
+
+        try
+        {
+            AprationalMath.doubleFactorial(-2);
+            fail("Double factorial of even negative number allowed");
+        }
+        catch (ApfloatArithmeticException aae)
+        {
+            // OK: double factorial of even negative number
+            assertEquals("Localization key", "doubleFactorial.ofNegativeEven", aae.getLocalizationKey());
+        }
+
+        try
+        {
+            AprationalMath.doubleFactorial(Long.MIN_VALUE);
+            fail("Double factorial of long min value");
+        }
+        catch (ApfloatArithmeticException aae)
+        {
+            // OK: double factorial of even negative number
+            assertEquals("Localization key", "doubleFactorial.ofNegativeEven", aae.getLocalizationKey());
+        }
+    }
+
     public static void testBinomial()
     {
         Aprational a = AprationalMath.binomial(new Aprational("0"), new Aprational("0"));
@@ -554,36 +602,40 @@ public class AprationalMathTest
             AprationalMath.binomial(new Aprational("9"), new Aprational("4/3"));
             fail("9,4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("9"), new Aprational("-4/3"));
             fail("9,-4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("-9"), new Aprational("4/3"));
             fail("-9,4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is infinite
+            assertEquals("Localization key", "binomial.infinite", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("-9"), new Aprational("-4/3"));
             fail("-9,-4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is infinite
+            assertEquals("Localization key", "binomial.infinite", aae.getLocalizationKey());
         }
 
         try
@@ -591,36 +643,40 @@ public class AprationalMathTest
             AprationalMath.binomial(new Aprational("2"), new Aprational("19/3"));
             fail("2,19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("2"), new Aprational("-19/3"));
             fail("2,-19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("-2"), new Aprational("19/3"));
             fail("-2,19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is infinite
+            assertEquals("Localization key", "binomial.infinite", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("-2"), new Aprational("-19/3"));
             fail("-2,-19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is infinite
+            assertEquals("Localization key", "binomial.infinite", aae.getLocalizationKey());
         }
 
         try
@@ -628,36 +684,40 @@ public class AprationalMathTest
             AprationalMath.binomial(new Aprational("-19/3"), new Aprational("4/3"));
             fail("-19/3,4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("19/3"), new Aprational("-4/3"));
             fail("19/3,-4/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("-4/3"), new Aprational("19/3"));
             fail("-4/3,19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
         try
         {
             AprationalMath.binomial(new Aprational("4/3"), new Aprational("-19/3"));
             fail("4/3,-19/3 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result is not a rational number
+            assertEquals("Localization key", "binomial.nonRational", aae.getLocalizationKey());
         }
 
         try
@@ -712,116 +772,6 @@ public class AprationalMathTest
         a = AprationalMath.bernoulli(6, 7);
         assertEquals("6 value", new Aprational("1/60", 7), a);
 
-        // This is *extremely* time-consuming
-        //assertEquals("Big equals small", AprationalMath.bernoulliSmall(2001, 10), AprationalMath.bernoulli(2001));
-
-        Iterator<Aprational> iterator = AprationalMath.bernoullis(4, 10);
-        assertEquals("iterator hasNext", true, iterator.hasNext());
-        assertEquals("iterator next 1", new Aprational("1"), iterator.next());
-        assertEquals("iterator next 2", new Aprational("-1/2"), iterator.next());
-        assertEquals("iterator next 3", new Aprational("1/6"), iterator.next());
-        assertEquals("iterator next 4", new Aprational("0"), iterator.next());
-        assertEquals("iterator next 5", new Aprational("-1/30"), iterator.next());
-
-        iterator = AprationalMath.bernoullisSmall(10);
-        assertEquals("iteratorSmall hasNext", true, iterator.hasNext());
-        assertEquals("iteratorSmall next 1", new Aprational("1"), iterator.next());
-        assertEquals("iteratorSmall next 2", new Aprational("-1/2"), iterator.next());
-        assertEquals("iteratorSmall next 3", new Aprational("1/6"), iterator.next());
-        assertEquals("iteratorSmall next 4", new Aprational("0"), iterator.next());
-        assertEquals("iteratorSmall next 5", new Aprational("-1/30"), iterator.next());
-        assertEquals("iteratorSmall hasNext 2", true, iterator.hasNext());
-
-        iterator = AprationalMath.bernoullisBig(4, 10);
-        assertEquals("iteratorBig hasNext", true, iterator.hasNext());
-        assertEquals("iteratorBig next 1", new Aprational("1"), iterator.next());
-        assertEquals("iteratorBig next 2", new Aprational("-1/2"), iterator.next());
-        assertEquals("iteratorBig next 3", new Aprational("1/6"), iterator.next());
-        assertEquals("iteratorBig next 4", new Aprational("0"), iterator.next());
-        assertEquals("iteratorBig next 5", new Aprational("-1/30"), iterator.next());
-        assertEquals("iteratorBig hasNext 2", false, iterator.hasNext());
-
-        try
-        {
-            iterator.next();
-            fail("iteratorBig allowed next");
-        }
-        catch (NoSuchElementException nsee)
-        {
-            // OK
-        }
-
-        iterator = AprationalMath.bernoullisSmall(2);
-        Iterator<Aprational> iterator2 = AprationalMath.bernoullisBig(100, 2);
-        for (int i = 0; i <= 100; i++)
-        {
-            a = (i > 0 ? AprationalMath.bernoulliSmall(i, 2) : AprationalMath.bernoulli(i, 2));
-            assertEquals("directSmall vs iteratorSmall " + i, a, iterator.next());
-            assertEquals("directSmall vs iteratorBig " + i, a, iterator2.next());
-            if (i % 2 == 0 && i > 0)
-            {
-                assertEquals("directSmall vs directBig " + i, a, AprationalMath.bernoulliBig(i, 2));
-            }
-        }
-
-        iterator = AprationalMath.bernoullis(4, 15);
-        assertEquals("iterator radix 15 next 1", new Aprational("1", 15), iterator.next());
-        assertEquals("iterator radix 15 next 2", new Aprational("-1/2", 15), iterator.next());
-        assertEquals("iterator radix 15 next 3", new Aprational("1/6", 15), iterator.next());
-        assertEquals("iterator radix 15 next 4", new Aprational("0", 15), iterator.next());
-        assertEquals("iterator radix 15 next 5", new Aprational("-1/20", 15), iterator.next());
-
-        iterator = AprationalMath.bernoullisSmall(15);
-        assertEquals("iteratorSmall radix 15 next 1", new Aprational("1", 15), iterator.next());
-        assertEquals("iteratorSmall radix 15 next 2", new Aprational("-1/2", 15), iterator.next());
-        assertEquals("iteratorSmall radix 15 next 3", new Aprational("1/6", 15), iterator.next());
-        assertEquals("iteratorSmall radix 15 next 4", new Aprational("0", 15), iterator.next());
-        assertEquals("iteratorSmall radix 15 next 5", new Aprational("-1/20", 15), iterator.next());
-
-        iterator = AprationalMath.bernoullisBig(4, 15);
-        assertEquals("iteratorBig radix 15 next 1", new Aprational("1", 15), iterator.next());
-        assertEquals("iteratorBig radix 15 next 2", new Aprational("-1/2", 15), iterator.next());
-        assertEquals("iteratorBig radix 15 next 3", new Aprational("1/6", 15), iterator.next());
-        assertEquals("iteratorBig radix 15 next 4", new Aprational("0", 15), iterator.next());
-        assertEquals("iteratorBig radix 15 next 5", new Aprational("-1/20", 15), iterator.next());
-
-        iterator = AprationalMath.bernoullis2(2, 10);
-        assertEquals("iterator2 hasNext", true, iterator.hasNext());
-        assertEquals("iterator2 next 1", new Aprational("1/6"), iterator.next());
-        assertEquals("iterator2 next 2", new Aprational("-1/30"), iterator.next());
-
-        iterator = AprationalMath.bernoullis2Small(10);
-        assertEquals("iterator2Small hasNext", true, iterator.hasNext());
-        assertEquals("iterator2Small next 1", new Aprational("1/6"), iterator.next());
-        assertEquals("iterator2Small next 2", new Aprational("-1/30"), iterator.next());
-        assertEquals("iterator2Small hasNext 2", true, iterator.hasNext());
-
-        iterator = AprationalMath.bernoullis2Big(2, 10);
-        assertEquals("iterator2Big hasNext", true, iterator.hasNext());
-        assertEquals("iterator2Big next 1", new Aprational("1/6"), iterator.next());
-        assertEquals("iterator2Big next 2", new Aprational("-1/30"), iterator.next());
-        assertEquals("iterator2Big hasNext 2", false, iterator.hasNext());
-
-        try
-        {
-            iterator.next();
-            fail("iterator2Big allowed next");
-        }
-        catch (NoSuchElementException nsee)
-        {
-            // OK
-        }
-
-        for (int radix = 2; radix <= 36; radix++)
-        {
-            iterator = AprationalMath.bernoullisSmall(radix);
-            iterator2 = AprationalMath.bernoullisBig(50, radix);
-            for (int i = 0; i <= 50; i++)
-            {
-                assertEquals("iteratorSmall vs iteratorBig " + i + " radix " + radix, iterator.next(), iterator2.next());
-            }
-        }
-
         try
         {
             AprationalMath.bernoulli(-1);
@@ -856,9 +806,10 @@ public class AprationalMathTest
             AprationalMath.harmonicNumber(new Apint(-1));
             fail("-1 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "harmonicNumber.negative", aae.getLocalizationKey());
         }
     }
 
@@ -897,9 +848,10 @@ public class AprationalMathTest
             AprationalMath.harmonicNumber(new Apint(-1), new Apint(2));
             fail("-1, 2 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "harmonicNumber.negative", aae.getLocalizationKey());
         }
     }
 }
