@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2024 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@ import java.util.IllegalFormatException;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.10.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -520,9 +520,10 @@ public class AprationalTest
             a.divide(new Aprational("0"));
             fail("Division by zero allowed");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: division by zero
+            assertEquals("Localization key", "divide.byZero", aae.getLocalizationKey());
         }
     }
 
@@ -852,8 +853,6 @@ public class AprationalTest
     public static void testFormatTo()
         throws IOException
     {
-        System.setProperty("java.locale.providers", "COMPAT,SPI"); // Required since Java 10 to have all locale providers available
-
         Locale locale = null;
         assertEquals("null %s", "123456789/1234", String.format(locale, "%s", new Aprational("123456789/1234")));
         assertEquals("null %S", "123456789/123A", String.format(locale, "%S", new Aprational("123456789/123a", 11)));
@@ -862,7 +861,7 @@ public class AprationalTest
 
         assertEquals("null %s apint", "123456789", String.format(locale, "%s", new Aprational("123456789")));
 
-        locale = new Locale("hi", "IN");
+        locale = new Locale.Builder().setLanguage("hi").setRegion("IN").setExtension('u', "nu-deva").build();
         assertEquals("hi_IN %s", "१२३४५६७८९/१२३४", String.format(locale, "%s", new Aprational("123456789/1234")));
         assertEquals("hi_IN %s radix 9", "१२३४५६७८/१२४", String.format(locale, "%s", new Aprational("12345678/124", 9)));
         assertEquals("hi_IN %s radix 11", "12345678/124", String.format(locale, "%s", new Aprational("12345678/124", 11)));
