@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@ package org.apfloat;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -127,6 +127,10 @@ public class FixedPrecisionApcomplexHelperTest
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testBesselI"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testBesselY"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testBesselK"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testStruveH"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testStruveL"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testAngerJ"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testWeberE"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testEllipticK"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testEllipticE"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testHermiteH"));
@@ -436,9 +440,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.pow(new Apcomplex(new Apfloat(0)), new Apcomplex(new Apfloat(0)));
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
 
         try
@@ -493,9 +498,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.atanh(new Apcomplex("1"));
             fail("atanh(1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "divide.byZero", aae.getLocalizationKey());
         }
 
         try
@@ -503,9 +509,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.atanh(new Apcomplex("-1"));
             fail("atanh(-1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "log.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -743,9 +750,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.logGamma(new Apcomplex("0"));
             fail("logGamma(0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "logGamma.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -762,9 +770,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.digamma(new Apcomplex("0"));
             fail("digamma(0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "digamma.ofNonpositiveInteger", aae.getLocalizationKey());
         }
     }
 
@@ -781,9 +790,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.polygamma(1, new Apcomplex("0"));
             fail("polygamma(1, 0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "polygamma.ofNonpositiveInteger", aae.getLocalizationKey());
         }
 
         try
@@ -791,9 +801,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.polygamma(-1, new Apcomplex("1"));
             fail("polygamma(-1, 1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "polygamma.ofNegativeOrder", aae.getLocalizationKey());
         }
     }
 
@@ -811,9 +822,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.beta(new Apcomplex("0"), new Apcomplex("0"));
             fail("beta(0, 0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "beta.infinite", aae.getLocalizationKey());
         }
     }
 
@@ -832,9 +844,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.beta(new Apcomplex("0"), new Apcomplex("0"), new Apcomplex("0"));
             fail("beta(0, 0, 0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "betaIncomplete.withNonpositiveInteger", aae.getLocalizationKey());
         }
     }
 
@@ -854,9 +867,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.beta(new Apcomplex("0"), new Apcomplex("1"), new Apcomplex("0"), new Apcomplex("0"));
             fail("beta(0, 1, 0, 0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "betaIncompleteGeneralized.withNonpositiveInteger", aae.getLocalizationKey());
         }
     }
 
@@ -874,9 +888,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.pochhammer(new Apcomplex("-0.5"), new Apcomplex("0.5"));
             fail("pochhammer(-0.5, 0.5) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "gamma.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -894,9 +909,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.binomial(new Apcomplex("-3"), new Apcomplex("(0,1)"));
             fail("binomial(-3,i) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "gamma.ofNegativeInteger", aae.getLocalizationKey());
         }
     }
 
@@ -1023,9 +1039,10 @@ public class FixedPrecisionApcomplexHelperTest
             helper.hypergeometricU(new Apcomplex("1"), new Apcomplex("2"), new Apcomplex("0"));
             fail("U(1, 2, 0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "pow.zeroToNegative", aae.getLocalizationKey());
         }
     }
 
@@ -1320,6 +1337,46 @@ public class FixedPrecisionApcomplexHelperTest
         assertEquals("precision", 6, result.precision());
     }
 
+    public static void testStruveH()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
+        Apcomplex ν = new Apcomplex("(1.2,3.4)"),
+                  z = new Apcomplex("(5.6,7.8)");
+        Apcomplex result = helper.struveH(ν, z);
+        assertEquals("value", new Apcomplex("(-1.42249,-2.92784)"), result, new Apfloat("5e-5"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testStruveL()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
+        Apcomplex ν = new Apcomplex("(1.2,3.4)"),
+                  z = new Apcomplex("(5.6,7.8)");
+        Apcomplex result = helper.struveL(ν, z);
+        assertEquals("value", new Apcomplex("(31.1937,11.0854)"), result, new Apfloat("5e-4"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testAngerJ()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
+        Apcomplex ν = new Apcomplex("(1.2,3.4)"),
+                  z = new Apcomplex("(5.6,7.8)");
+        Apcomplex result = helper.angerJ(ν, z);
+        assertEquals("value", new Apcomplex("(-531.370,42.5825)"), result, new Apfloat("5e-4"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testWeberE()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
+        Apcomplex ν = new Apcomplex("(1.2,3.4)"),
+                  z = new Apcomplex("(5.6,7.8)");
+        Apcomplex result = helper.weberE(ν, z);
+        assertEquals("value", new Apcomplex("(-41.0167,-525.327)"), result, new Apfloat("5e-4"));
+        assertEquals("precision", 6, result.precision());
+    }
+
     public static void testEllipticK()
     {
         FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
