@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2024 Mikko Tommila
+ * Copyright (c) 2002-2025 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@ import java.math.RoundingMode;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.14.0
+ * @version 1.15.0
  * @author Mikko Tommila
  */
 
@@ -153,6 +153,10 @@ public class FixedPrecisionApfloatHelperTest
         suite.addTest(new FixedPrecisionApfloatHelperTest("testBesselI"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testBesselY"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testBesselK"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testStruveH"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testStruveL"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testAngerJ"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testWeberE"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testEllipticK"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testEllipticE"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testHermiteH"));
@@ -250,9 +254,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.pow(new Apfloat(0), 0);
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
     }
 
@@ -278,9 +283,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.inverseRoot(x, 0);
             fail("inverse zeroth root accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: inverse zeroth root
+            assertEquals("Localization key", "inverseRoot.zeroth", aae.getLocalizationKey());
         }
 
         try
@@ -288,9 +294,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.inverseRoot(new Apfloat(-2), 2);
             fail("inverse sqrt of -2 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be imaginary
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
 
         try
@@ -298,9 +305,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.inverseRoot(new Apfloat(0), 2);
             fail("inverse sqrt of 0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be infinite
+            assertEquals("Localization key", "inverseRoot.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -317,9 +325,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.root(x, 0);
             fail("zeroth root accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: zeroth root
+            assertEquals("Localization key", "root.zeroth", aae.getLocalizationKey());
         }
 
         try
@@ -327,9 +336,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.root(new Apfloat(-2), 2);
             fail("sqrt of -2 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be imaginary
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
 
         try
@@ -337,9 +347,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.root(new Apfloat(0), 0);
             fail("0th root of 0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK: result would be undefined
+            assertEquals("Localization key", "root.zeroth", aae.getLocalizationKey());
         }
     }
 
@@ -576,6 +587,14 @@ public class FixedPrecisionApfloatHelperTest
         assertEquals("value 16", new Apfloat(0xF00, Apfloat.DEFAULT, 16), result);
         assertEquals("precision 16", 10, result.precision());
 
+        result = helper.doubleFactorial(-5);
+        assertEquals("value -5", new Apfloat("0.3333333333"), result, new Apfloat("1e-10"));
+        assertEquals("precision -5", 10, result.precision());
+
+        result = helper.doubleFactorial(-7, 16);
+        assertEquals("value -7", new Apfloat(-1.0 / 15.0, Apfloat.DEFAULT, 16), result, new Apfloat("0.0000000001", 1, 16));
+        assertEquals("precision -7", 10, result.precision());
+
         try
         {
             helper.doubleFactorial(7, 1);
@@ -585,6 +604,17 @@ public class FixedPrecisionApfloatHelperTest
         {
             // OK; invalid radix
         }
+
+        try
+        {
+            helper.doubleFactorial(-2, 16);
+            fail("-2 accepted");
+        }
+        catch (ApfloatArithmeticException aae)
+        {
+            // OK
+            assertEquals("Localization key", "doubleFactorial.ofNegativeEven", aae.getLocalizationKey());
+        }
     }
 
     public static void testPi()
@@ -645,9 +675,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.log(new Apfloat(0));
             fail("log of zero accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be -infinite
+            assertEquals("Localization key", "log.ofZero", aae.getLocalizationKey());
         }
 
         try
@@ -655,9 +686,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.log(new Apfloat(-1));
             fail("log of -1 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "log.ofNegative", aae.getLocalizationKey());
         }
     }
 
@@ -738,9 +770,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.pow(new Apfloat(0), new Apfloat(0));
             fail("0^0 accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "pow.zeroToZero", aae.getLocalizationKey());
         }
 
         try
@@ -758,9 +791,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.pow(new Apfloat(-2), new Apfloat("1.3"));
             fail("pow of negative number accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "pow.negativeToNonInteger", aae.getLocalizationKey());
         }
     }
 
@@ -777,9 +811,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.acosh(new Apfloat("0.9"));
             fail("acosh(0.9) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
 
         try
@@ -787,9 +822,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.acosh(new Apfloat("-0.5"));
             fail("acosh(-0.5) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
     }
 
@@ -815,9 +851,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.atanh(new Apfloat("1"));
             fail("atanh(1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "divide.byZero", aae.getLocalizationKey());
         }
 
         try
@@ -825,9 +862,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.atanh(new Apfloat("-1"));
             fail("atanh(-1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be infinite
+            assertEquals("Localization key", "log.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -890,9 +928,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.acos(new Apfloat("1.1"));
             fail("acos(1.1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
     }
 
@@ -909,9 +948,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.asin(new Apfloat("1.1"));
             fail("asin(1.1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be complex
+            assertEquals("Localization key", "root.evenOfNegative", aae.getLocalizationKey());
         }
     }
 
@@ -937,9 +977,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.atan2(new Apfloat(0), new Apfloat(0));
             fail("atan2(0,0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK; result would be undefined
+            assertEquals("Localization key", "atan2.ofOrigin", aae.getLocalizationKey());
         }
     }
 
@@ -1237,9 +1278,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.logGamma(new Apfloat(0));
             fail("logGamma(0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "logGamma.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -1256,9 +1298,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.digamma(new Apfloat(0));
             fail("digamma(0) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "digamma.ofNonpositiveInteger", aae.getLocalizationKey());
         }
     }
 
@@ -1296,9 +1339,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.beta(new Apfloat("1.5"), new Apfloat("-1.5"), new Apfloat("2.5"));
             fail("beta(1.5, -1.5, 2.5) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "complex", aae.getLocalizationKey());
         }
     }
 
@@ -1346,9 +1390,10 @@ public class FixedPrecisionApfloatHelperTest
             helper.binomial(new Apfloat(-1), new Apfloat("-1.1"));
             fail("binomial(-1,-1.1) accepted");
         }
-        catch (ArithmeticException ae)
+        catch (ApfloatArithmeticException aae)
         {
             // OK
+            assertEquals("Localization key", "gamma.ofZero", aae.getLocalizationKey());
         }
     }
 
@@ -1729,6 +1774,13 @@ public class FixedPrecisionApfloatHelperTest
         Apfloat result = helper.besselJ(ν, x);
         assertEquals("value", new Apfloat("-0.140294"), result, new Apfloat("5e-6"));
         assertEquals("precision", 6, result.precision());
+
+        helper = new FixedPrecisionApfloatHelper(17);
+        ν = new Apfloat("-1.0");
+        x = new Apfloat("-1009.0");
+        result = helper.besselJ(ν, x);
+        assertEquals("-1 value", new Apfloat("0.0058796558493124600"), result, new Apfloat("5e-19"));
+        assertEquals("-1 precision", 17, result.precision());
     }
 
     public static void testBesselI()
@@ -1761,6 +1813,46 @@ public class FixedPrecisionApfloatHelperTest
         assertEquals("precision", 6, result.precision());
     }
 
+    public static void testStruveH()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
+        Apfloat ν = new Apfloat("-1.5"),
+                x = new Apfloat("2.5");
+        Apfloat result = helper.struveH(ν, x);
+        assertEquals("value", new Apfloat("-0.525080"), result, new Apfloat("5e-6"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testStruveL()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
+        Apfloat ν = new Apfloat("-1.5"),
+                x = new Apfloat("2.5");
+        Apfloat result = helper.struveL(ν, x);
+        assertEquals("value", new Apfloat("1.87328"), result, new Apfloat("5e-5"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testAngerJ()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
+        Apfloat ν = new Apfloat("-1.5"),
+                x = new Apfloat("2.5");
+        Apfloat result = helper.angerJ(ν, x);
+        assertEquals("value", new Apfloat("0.0698371"), result, new Apfloat("5e-7"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testWeberE()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
+        Apfloat ν = new Apfloat("-1.5"),
+                x = new Apfloat("2.5");
+        Apfloat result = helper.weberE(ν, x);
+        assertEquals("value", new Apfloat("0.447994"), result, new Apfloat("5e-6"));
+        assertEquals("precision", 6, result.precision());
+    }
+
     public static void testEllipticK()
     {
         FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
