@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -75,7 +75,7 @@ public class RawtypeBaseMathTest
         int size = data.length;
         ApfloatContext ctx = ApfloatContext.getContext();
         DataStorageBuilder dataStorageBuilder = ctx.getBuilderFactory().getDataStorageBuilder();
-        DataStorage dataStorage = dataStorageBuilder.createDataStorage(size * sizeof(rawtype));
+        DataStorage dataStorage = dataStorageBuilder.createDataStorage(size * RawType.BYTES);
         dataStorage.setSize(size);
 
         try (ArrayAccess arrayAccess = dataStorage.getArray(DataStorage.WRITE, 0, size))
